@@ -30,7 +30,6 @@
 
 #include "libavutil/intreadwrite.h"
 #include "cabac.h"
-#include "dsputil.h"
 #include "error_resilience.h"
 #include "get_bits.h"
 #include "h264chroma.h"
@@ -38,6 +37,7 @@
 #include "h264pred.h"
 #include "h264qpel.h"
 #include "internal.h" // for avpriv_find_start_code()
+#include "me_cmp.h"
 #include "mpegutils.h"
 #include "parser.h"
 #include "qpeldsp.h"
@@ -338,15 +338,14 @@ typedef struct H264Picture {
  * H264Context
  */
 typedef struct H264Context {
-    AVClass *av_class;
     AVCodecContext *avctx;
+    MECmpContext mecc;
     VideoDSPContext vdsp;
     H264DSPContext h264dsp;
     H264ChromaContext h264chroma;
     H264QpelContext h264qpel;
     ParseContext parse_context;
     GetBitContext gb;
-    DSPContext       dsp;
     ERContext er;
 
     H264Picture *DPB;
