@@ -25,9 +25,11 @@
 
 #include "libavutil/bswap.h"
 #include "libavutil/common.h"
+#include "libavutil/intreadwrite.h"
 #include "libavutil/lzo.h"
 #include "libavutil/imgutils.h"
 #include "avcodec.h"
+#include "idctdsp.h"
 #include "internal.h"
 #include "rtjpeg.h"
 
@@ -40,7 +42,6 @@ typedef struct {
     unsigned char *decomp_buf;
     uint32_t lq[64], cq[64];
     RTJpegContext rtj;
-    DSPContext dsp;
 } NuvContext;
 
 static const uint8_t fallback_lquant[] = {
@@ -138,13 +139,11 @@ static int codec_reinit(AVCodecContext *avctx, int width, int height,
                    "Can't allocate decompression buffer.\n");
             return AVERROR(ENOMEM);
         }
-        ff_rtjpeg_decode_init(&c->rtj, &c->dsp, c->width, c->height,
-                              c->lq, c->cq);
+        ff_rtjpeg_decode_init(&c->rtj, c->width, c->height, c->lq, c->cq);
         av_frame_unref(c->pic);
         return 1;
     } else if (quality != c->quality)
-        ff_rtjpeg_decode_init(&c->rtj, &c->dsp, c->width, c->height,
-                              c->lq, c->cq);
+        ff_rtjpeg_decode_init(&c->rtj, c->width, c->height, c->lq, c->cq);
 
     return 0;
 }
@@ -183,8 +182,7 @@ static int decode_frame(AVCodecContext *avctx, void *data, int *got_frame,
         ret       = get_quant(avctx, c, buf, buf_size);
         if (ret < 0)
             return ret;
-        ff_rtjpeg_decode_init(&c->rtj, &c->dsp, c->width, c->height, c->lq,
-                              c->cq);
+        ff_rtjpeg_decode_init(&c->rtj, c->width, c->height, c->lq, c->cq);
         return orig_size;
     }
 
@@ -322,7 +320,7 @@ static av_cold int decode_init(AVCodecContext *avctx)
     if (avctx->extradata_size)
         get_quant(avctx, c, avctx->extradata, avctx->extradata_size);
 
-    ff_dsputil_init(&c->dsp, avctx);
+    ff_rtjpeg_init(&c->rtj, avctx);
 
     if ((ret = codec_reinit(avctx, avctx->width, avctx->height, -1)) < 0)
         return ret;
