@@ -617,12 +617,9 @@ static int svq1_decode_frame(AVCodecContext *avctx, void *data,
     uint8_t *current;
     int result, i, x, y, width, height;
     svq1_pmv *pmv;
-    int ret;
 
     /* initialize bit buffer */
-    ret = init_get_bits8(&s->gb, buf, buf_size);
-    if (ret < 0)
-        return ret;
+    init_get_bits8(&s->gb, buf, buf_size);
 
     /* decode frame header */
     s->frame_code = get_bits(&s->gb, 22);
@@ -639,9 +636,8 @@ static int svq1_decode_frame(AVCodecContext *avctx, void *data,
             return AVERROR_INVALIDDATA;
         }
 
-        av_fast_padded_malloc(&s->pkt_swapped,
-                              &s->pkt_swapped_allocated,
-                              buf_size);
+        av_fast_padded_malloc(&s->pkt_swapped, &s->pkt_swapped_allocated,
+                       buf_size);
         if (!s->pkt_swapped)
             return AVERROR(ENOMEM);
 
