@@ -166,7 +166,7 @@ static int svq3_decode_end(AVCodecContext *avctx);
 
 void ff_svq3_luma_dc_dequant_idct_c(int16_t *output, int16_t *input, int qp)
 {
-    const unsigned qmul = svq3_dequant_coeff[qp];
+    const int qmul = svq3_dequant_coeff[qp];
 #define stride 16
     int i;
     int temp[16];
@@ -191,10 +191,10 @@ void ff_svq3_luma_dc_dequant_idct_c(int16_t *output, int16_t *input, int qp)
         const int z2     =  7 *  temp[4 * 1 + i] - 17 * temp[4 * 3 + i];
         const int z3     = 17 *  temp[4 * 1 + i] +  7 * temp[4 * 3 + i];
 
-        output[stride *  0 + offset] = (int)((z0 + z3) * qmul + 0x80000) >> 20;
-        output[stride *  2 + offset] = (int)((z1 + z2) * qmul + 0x80000) >> 20;
-        output[stride *  8 + offset] = (int)((z1 - z2) * qmul + 0x80000) >> 20;
-        output[stride * 10 + offset] = (int)((z0 - z3) * qmul + 0x80000) >> 20;
+        output[stride *  0 + offset] = (z0 + z3) * qmul + 0x80000 >> 20;
+        output[stride *  2 + offset] = (z1 + z2) * qmul + 0x80000 >> 20;
+        output[stride *  8 + offset] = (z1 - z2) * qmul + 0x80000 >> 20;
+        output[stride * 10 + offset] = (z0 - z3) * qmul + 0x80000 >> 20;
     }
 }
 #undef stride
@@ -206,7 +206,7 @@ void ff_svq3_add_idct_c(uint8_t *dst, int16_t *block,
     int i;
 
     if (dc) {
-        dc       = 13 * 13 * (dc == 1 ? 1538U* block[0]
+        dc       = 13 * 13 * (dc == 1 ? 1538 * block[0]
                                       : qmul * (block[0] >> 3) / 2);
         block[0] = 0;
     }
@@ -224,16 +224,16 @@ void ff_svq3_add_idct_c(uint8_t *dst, int16_t *block,
     }
 
     for (i = 0; i < 4; i++) {
-        const unsigned z0 = 13 * (block[i + 4 * 0] +      block[i + 4 * 2]);
-        const unsigned z1 = 13 * (block[i + 4 * 0] -      block[i + 4 * 2]);
-        const unsigned z2 =  7 *  block[i + 4 * 1] - 17 * block[i + 4 * 3];
-        const unsigned z3 = 17 *  block[i + 4 * 1] +  7 * block[i + 4 * 3];
-        const int rr = (dc + 0x80000u);
+        const int z0 = 13 * (block[i + 4 * 0] +      block[i + 4 * 2]);
+        const int z1 = 13 * (block[i + 4 * 0] -      block[i + 4 * 2]);
+        const int z2 =  7 *  block[i + 4 * 1] - 17 * block[i + 4 * 3];
+        const int z3 = 17 *  block[i + 4 * 1] +  7 * block[i + 4 * 3];
+        const int rr = (dc + 0x80000);
 
-        dst[i + stride * 0] = av_clip_uint8(dst[i + stride * 0] + ((int)((z0 + z3) * qmul + rr) >> 20));
-        dst[i + stride * 1] = av_clip_uint8(dst[i + stride * 1] + ((int)((z1 + z2) * qmul + rr) >> 20));
-        dst[i + stride * 2] = av_clip_uint8(dst[i + stride * 2] + ((int)((z1 - z2) * qmul + rr) >> 20));
-        dst[i + stride * 3] = av_clip_uint8(dst[i + stride * 3] + ((int)((z0 - z3) * qmul + rr) >> 20));
+        dst[i + stride * 0] = av_clip_uint8(dst[i + stride * 0] + ((z0 + z3) * qmul + rr >> 20));
+        dst[i + stride * 1] = av_clip_uint8(dst[i + stride * 1] + ((z1 + z2) * qmul + rr >> 20));
+        dst[i + stride * 2] = av_clip_uint8(dst[i + stride * 2] + ((z1 - z2) * qmul + rr >> 20));
+        dst[i + stride * 3] = av_clip_uint8(dst[i + stride * 3] + ((z0 - z3) * qmul + rr >> 20));
     }
 
     memset(block, 0, 16 * sizeof(int16_t));
@@ -395,8 +395,8 @@ static inline int svq3_mc_dir(SVQ3Context *s, int size, int mode,
             if (mode != PREDICT_MODE) {
                 pred_motion(h, k, part_width >> 2, dir, 1, &mx, &my);
             } else {
-                mx = s->next_pic->motion_val[0][b_xy][0] * 2;
-                my = s->next_pic->motion_val[0][b_xy][1] * 2;
+                mx = s->next_pic->motion_val[0][b_xy][0] << 1;
+                my = s->next_pic->motion_val[0][b_xy][1] << 1;
 
                 if (dir == 0) {
                     mx = mx * h->frame_num_offset /
@@ -433,8 +433,8 @@ static inline int svq3_mc_dir(SVQ3Context *s, int size, int mode,
                 int fx, fy;
                 mx  = (mx + 1 >> 1) + dx;
                 my  = (my + 1 >> 1) + dy;
-                fx  = (unsigned)(mx + 0x30000) / 3 - 0x10000;
-                fy  = (unsigned)(my + 0x30000) / 3 - 0x10000;
+                fx  = (unsigned)(mx + 0x3000) / 3 - 0x1000;
+                fy  = (unsigned)(my + 0x3000) / 3 - 0x1000;
                 dxy = (mx - 3 * fx) + 4 * (my - 3 * fy);
 
                 svq3_mc_dir_part(s, x, y, part_width, part_height,
@@ -442,8 +442,8 @@ static inline int svq3_mc_dir(SVQ3Context *s, int size, int mode,
                 mx += mx;
                 my += my;
             } else if (mode == HALFPEL_MODE || mode == PREDICT_MODE) {
-                mx  = (unsigned)(mx + 1 + 0x30000) / 3 + dx - 0x10000;
-                my  = (unsigned)(my + 1 + 0x30000) / 3 + dy - 0x10000;
+                mx  = (unsigned)(mx + 1 + 0x3000) / 3 + dx - 0x1000;
+                my  = (unsigned)(my + 1 + 0x3000) / 3 + dy - 0x1000;
                 dxy = (mx & 1) + 2 * (my & 1);
 
                 svq3_mc_dir_part(s, x, y, part_width, part_height,
@@ -451,8 +451,8 @@ static inline int svq3_mc_dir(SVQ3Context *s, int size, int mode,
                 mx *= 3;
                 my *= 3;
             } else {
-                mx = (unsigned)(mx + 3 + 0x60000) / 6 + dx - 0x10000;
-                my = (unsigned)(my + 3 + 0x60000) / 6 + dy - 0x10000;
+                mx = (unsigned)(mx + 3 + 0x6000) / 6 + dx - 0x1000;
+                my = (unsigned)(my + 3 + 0x6000) / 6 + dy - 0x1000;
 
                 svq3_mc_dir_part(s, x, y, part_width, part_height,
                                  mx, my, 0, 0, dir, avg);
