@@ -216,12 +216,12 @@ static int interp_##nbits##_##name(AVFilterContext *ctx, void *arg, int jobnr, i
     const int slice_end   = (in->height * (jobnr+1)) / nb_jobs;                                     \
     uint8_t       *dstrow = out->data[0] + slice_start * out->linesize[0];                          \
     const uint8_t *srcrow = in ->data[0] + slice_start * in ->linesize[0];                          \
+    const float scale = (1. / ((1<<nbits) - 1)) * (lut3d->lutsize - 1);                             \
                                                                                                     \
     for (y = slice_start; y < slice_end; y++) {                                                     \
         uint##nbits##_t *dst = (uint##nbits##_t *)dstrow;                                           \
         const uint##nbits##_t *src = (const uint##nbits##_t *)srcrow;                               \
         for (x = 0; x < in->width * step; x += step) {                                              \
-            const float scale = (1. / ((1<<nbits) - 1)) * (lut3d->lutsize - 1);                     \
             const struct rgbvec scaled_rgb = {src[x + r] * scale,                                   \
                                               src[x + g] * scale,                                   \
                                               src[x + b] * scale};                                  \
@@ -262,21 +262,34 @@ static int skip_line(const char *p)
     }                                                       \
 } while (loop_cond)
 
-/* Basically r g and b float values on each line; seems to be generated by
- * Davinci */
+/* Basically r g and b float values on each line, with a facultative 3DLUTSIZE
+ * directive; seems to be generated by Davinci */
 static int parse_dat(AVFilterContext *ctx, FILE *f)
 {
     LUT3DContext *lut3d = ctx->priv;
-    const int size = lut3d->lutsize;
-    int i, j, k;
+    char line[MAX_LINE_SIZE];
+    int i, j, k, size;
 
+    lut3d->lutsize = size = 33;
+
+    NEXT_LINE(skip_line(line));
+    if (!strncmp(line, "3DLUTSIZE ", 10)) {
+        size = strtol(line + 10, NULL, 0);
+        if (size < 2 || size > MAX_LEVEL) {
+            av_log(ctx, AV_LOG_ERROR, "Too large or invalid 3D LUT size\n");
+            return AVERROR(EINVAL);
+        }
+        lut3d->lutsize = size;
+        NEXT_LINE(skip_line(line));
+    }
     for (k = 0; k < size; k++) {
         for (j = 0; j < size; j++) {
             for (i = 0; i < size; i++) {
-                char line[MAX_LINE_SIZE];
                 struct rgbvec *vec = &lut3d->lut[k][j][i];
-                NEXT_LINE(skip_line(line));
-                sscanf(line, "%f %f %f", &vec->r, &vec->g, &vec->b);
+                if (k != 0 || j != 0 || i != 0)
+                    NEXT_LINE(skip_line(line));
+                if (sscanf(line, "%f %f %f", &vec->r, &vec->g, &vec->b) != 3)
+                    return AVERROR_INVALIDDATA;
             }
         }
     }
@@ -569,7 +582,6 @@ static av_cold int lut3d_init(AVFilterContext *ctx)
     ext++;
 
     if (!av_strcasecmp(ext, "dat")) {
-        lut3d->lutsize = 33;
         ret = parse_dat(ctx, f);
     } else if (!av_strcasecmp(ext, "3dl")) {
         ret = parse_3dl(ctx, f);
@@ -642,7 +654,7 @@ static void update_clut(LUT3DContext *lut3d, const AVFrame *frame)
             for (i = 0; i < level; i++) {                               \
                 const uint##nbits##_t *src = (const uint##nbits##_t *)  \
                     (data + y*linesize + x*step);                       \
-                struct rgbvec *vec = &lut3d->lut[k][j][i];              \
+                struct rgbvec *vec = &lut3d->lut[i][j][k];              \
                 vec->r = src[rgba_map[0]] / (float)((1<<(nbits)) - 1);  \
                 vec->g = src[rgba_map[1]] / (float)((1<<(nbits)) - 1);  \
                 vec->b = src[rgba_map[2]] / (float)((1<<(nbits)) - 1);  \
