@@ -36,7 +36,9 @@
 #include "internal.h"
 #include "video.h"
 
-static const char *var_names[] = {
+#include <float.h>
+
+static const char * const var_names[] = {
     "in_w" , "iw",  ///< width of the input video
     "in_h" , "ih",  ///< height of the input video
     "out_w", "ow",  ///< width of the input video
@@ -77,6 +79,16 @@ typedef struct {
     FFDrawColor color;
 } RotContext;
 
+typedef struct ThreadData {
+    AVFrame *in, *out;
+    int inw,  inh;
+    int outw, outh;
+    int plane;
+    int xi, yi;
+    int xprime, yprime;
+    int c, s;
+} ThreadData;
+
 #define OFFSET(x) offsetof(RotContext, x)
 #define FLAGS AV_OPT_FLAG_FILTERING_PARAM|AV_OPT_FLAG_VIDEO_PARAM
 
@@ -86,7 +98,7 @@ static const AVOption rotate_options[] = {
     { "out_w",     "set output width expression",  OFFSET(outw_expr_str), AV_OPT_TYPE_STRING, {.str="iw"}, CHAR_MIN, CHAR_MAX, .flags=FLAGS },
     { "ow",        "set output width expression",  OFFSET(outw_expr_str), AV_OPT_TYPE_STRING, {.str="iw"}, CHAR_MIN, CHAR_MAX, .flags=FLAGS },
     { "out_h",     "set output height expression", OFFSET(outh_expr_str), AV_OPT_TYPE_STRING, {.str="ih"}, CHAR_MIN, CHAR_MAX, .flags=FLAGS },
-    { "oh",        "set output width expression",  OFFSET(outh_expr_str), AV_OPT_TYPE_STRING, {.str="ih"}, CHAR_MIN, CHAR_MAX, .flags=FLAGS },
+    { "oh",        "set output height expression", OFFSET(outh_expr_str), AV_OPT_TYPE_STRING, {.str="ih"}, CHAR_MIN, CHAR_MAX, .flags=FLAGS },
     { "fillcolor", "set background fill color",    OFFSET(fillcolor_str), AV_OPT_TYPE_STRING, {.str="black"}, CHAR_MIN, CHAR_MAX, .flags=FLAGS },
     { "c",         "set background fill color",    OFFSET(fillcolor_str), AV_OPT_TYPE_STRING, {.str="black"}, CHAR_MIN, CHAR_MAX, .flags=FLAGS },
     { "bilinear",  "use bilinear interpolation",   OFFSET(use_bilinear),  AV_OPT_TYPE_INT, {.i64=1}, 0, 1, .flags=FLAGS },
@@ -118,7 +130,7 @@ static av_cold void uninit(AVFilterContext *ctx)
 
 static int query_formats(AVFilterContext *ctx)
 {
-    static enum PixelFormat pix_fmts[] = {
+    static const enum PixelFormat pix_fmts[] = {
         AV_PIX_FMT_GBRP,   AV_PIX_FMT_GBRAP,
         AV_PIX_FMT_ARGB,   AV_PIX_FMT_RGBA,
         AV_PIX_FMT_ABGR,   AV_PIX_FMT_BGRA,
@@ -242,11 +254,12 @@ static int config_props(AVFilterLink *outlink)
 }
 
 #define FIXP (1<<16)
-#define INT_PI 205887 //(M_PI * FIXP)
+#define FIXP2 (1<<20)
+#define INT_PI 3294199 //(M_PI * FIXP2)
 
 /**
  * Compute the sin of a using integer values.
- * Input and output values are scaled by FIXP.
+ * Input is scaled by FIXP2 and output values are scaled by FIXP.
  */
 static int64_t int_sin(int64_t a)
 {
@@ -258,13 +271,13 @@ static int64_t int_sin(int64_t a)
     if (a >= INT_PI*3/2) a -= 2*INT_PI;  // -PI/2 .. 3PI/2
     if (a >= INT_PI/2  ) a = INT_PI - a; // -PI/2 ..  PI/2
 
-    /* compute sin using Taylor series approximated to the third term */
-    a2 = (a*a)/FIXP;
-    for (i = 2; i < 7; i += 2) {
+    /* compute sin using Taylor series approximated to the fifth term */
+    a2 = (a*a)/(FIXP2);
+    for (i = 2; i < 11; i += 2) {
         res += a;
-        a = -a*a2 / (FIXP*i*(i+1));
+        a = -a*a2 / (FIXP2*i*(i+1));
     }
-    return res;
+    return (res + 8)>>4;
 }
 
 /**
@@ -297,8 +310,153 @@ static uint8_t *interpolate_bilinear(uint8_t *dst_color,
     return dst_color;
 }
 
+static av_always_inline void copy_elem(uint8_t *pout, const uint8_t *pin, int elem_size)
+{
+    int v;
+    switch (elem_size) {
+    case 1:
+        *pout = *pin;
+        break;
+    case 2:
+        *((uint16_t *)pout) = *((uint16_t *)pin);
+        break;
+    case 3:
+        v = AV_RB24(pin);
+        AV_WB24(pout, v);
+        break;
+    case 4:
+        *((uint32_t *)pout) = *((uint32_t *)pin);
+        break;
+    default:
+        memcpy(pout, pin, elem_size);
+        break;
+    }
+}
+
+static av_always_inline void simple_rotate_internal(uint8_t *dst, const uint8_t *src, int src_linesize, int angle, int elem_size, int len)
+{
+    int i;
+    switch(angle) {
+    case 0:
+        memcpy(dst, src, elem_size * len);
+        break;
+    case 1:
+        for (i = 0; i<len; i++)
+            copy_elem(dst + i*elem_size, src + (len-i-1)*src_linesize, elem_size);
+        break;
+    case 2:
+        for (i = 0; i<len; i++)
+            copy_elem(dst + i*elem_size, src + (len-i-1)*elem_size, elem_size);
+        break;
+    case 3:
+        for (i = 0; i<len; i++)
+            copy_elem(dst + i*elem_size, src + i*src_linesize, elem_size);
+        break;
+    }
+}
+
+static av_always_inline void simple_rotate(uint8_t *dst, const uint8_t *src, int src_linesize, int angle, int elem_size, int len)
+{
+    switch(elem_size) {
+    case 1 : simple_rotate_internal(dst, src, src_linesize, angle, 1, len); break;
+    case 2 : simple_rotate_internal(dst, src, src_linesize, angle, 2, len); break;
+    case 3 : simple_rotate_internal(dst, src, src_linesize, angle, 3, len); break;
+    case 4 : simple_rotate_internal(dst, src, src_linesize, angle, 4, len); break;
+    default: simple_rotate_internal(dst, src, src_linesize, angle, elem_size, len); break;
+    }
+}
+
 #define TS2T(ts, tb) ((ts) == AV_NOPTS_VALUE ? NAN : (double)(ts)*av_q2d(tb))
 
+static int filter_slice(AVFilterContext *ctx, void *arg, int job, int nb_jobs)
+{
+    ThreadData *td = arg;
+    AVFrame *in = td->in;
+    AVFrame *out = td->out;
+    RotContext *rot = ctx->priv;
+    const int outw = td->outw, outh = td->outh;
+    const int inw = td->inw, inh = td->inh;
+    const int plane = td->plane;
+    const int xi = td->xi, yi = td->yi;
+    const int c = td->c, s = td->s;
+    const int start = (outh *  job   ) / nb_jobs;
+    const int end   = (outh * (job+1)) / nb_jobs;
+    int xprime = td->xprime + start * s;
+    int yprime = td->yprime + start * c;
+    int i, j, x, y;
+
+    for (j = start; j < end; j++) {
+        x = xprime + xi + FIXP*(inw-1)/2;
+        y = yprime + yi + FIXP*(inh-1)/2;
+
+        if (fabs(rot->angle - 0) < FLT_EPSILON && outw == inw && outh == inh) {
+            simple_rotate(out->data[plane] + j * out->linesize[plane],
+                           in->data[plane] + j *  in->linesize[plane],
+                          in->linesize[plane], 0, rot->draw.pixelstep[plane], outw);
+        } else if (fabs(rot->angle - M_PI/2) < FLT_EPSILON && outw == inh && outh == inw) {
+            simple_rotate(out->data[plane] + j * out->linesize[plane],
+                           in->data[plane] + j * rot->draw.pixelstep[plane],
+                          in->linesize[plane], 1, rot->draw.pixelstep[plane], outw);
+        } else if (fabs(rot->angle - M_PI) < FLT_EPSILON && outw == inw && outh == inh) {
+            simple_rotate(out->data[plane] + j * out->linesize[plane],
+                           in->data[plane] + (outh-j-1) *  in->linesize[plane],
+                          in->linesize[plane], 2, rot->draw.pixelstep[plane], outw);
+        } else if (fabs(rot->angle - 3*M_PI/2) < FLT_EPSILON && outw == inh && outh == inw) {
+            simple_rotate(out->data[plane] + j * out->linesize[plane],
+                           in->data[plane] + (outh-j-1) * rot->draw.pixelstep[plane],
+                          in->linesize[plane], 3, rot->draw.pixelstep[plane], outw);
+        } else {
+
+        for (i = 0; i < outw; i++) {
+            int32_t v;
+            int x1, y1;
+            uint8_t *pin, *pout;
+            x1 = x>>16;
+            y1 = y>>16;
+
+            /* the out-of-range values avoid border artifacts */
+            if (x1 >= -1 && x1 <= inw && y1 >= -1 && y1 <= inh) {
+                uint8_t inp_inv[4]; /* interpolated input value */
+                pout = out->data[plane] + j * out->linesize[plane] + i * rot->draw.pixelstep[plane];
+                if (rot->use_bilinear) {
+                    pin = interpolate_bilinear(inp_inv,
+                                               in->data[plane], in->linesize[plane], rot->draw.pixelstep[plane],
+                                               x, y, inw-1, inh-1);
+                } else {
+                    int x2 = av_clip(x1, 0, inw-1);
+                    int y2 = av_clip(y1, 0, inh-1);
+                    pin = in->data[plane] + y2 * in->linesize[plane] + x2 * rot->draw.pixelstep[plane];
+                }
+                switch (rot->draw.pixelstep[plane]) {
+                case 1:
+                    *pout = *pin;
+                    break;
+                case 2:
+                    *((uint16_t *)pout) = *((uint16_t *)pin);
+                    break;
+                case 3:
+                    v = AV_RB24(pin);
+                    AV_WB24(pout, v);
+                    break;
+                case 4:
+                    *((uint32_t *)pout) = *((uint32_t *)pin);
+                    break;
+                default:
+                    memcpy(pout, pin, rot->draw.pixelstep[plane]);
+                    break;
+                }
+            }
+            x += c;
+            y -= s;
+        }
+        }
+        xprime += s;
+        yprime += c;
+    }
+
+    return 0;
+}
+
 static int filter_frame(AVFilterLink *inlink, AVFrame *in)
 {
     AVFilterContext *ctx = inlink->dst;
@@ -322,7 +480,7 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *in)
     av_log(ctx, AV_LOG_DEBUG, "n:%f time:%f angle:%f/PI\n",
            rot->var_values[VAR_N], rot->var_values[VAR_T], rot->angle/M_PI);
 
-    angle_int = res * FIXP;
+    angle_int = res * FIXP * 16;
     s = int_sin(angle_int);
     c = int_sin(angle_int + INT_PI/2);
 
@@ -334,66 +492,19 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *in)
     for (plane = 0; plane < rot->nb_planes; plane++) {
         int hsub = plane == 1 || plane == 2 ? rot->hsub : 0;
         int vsub = plane == 1 || plane == 2 ? rot->vsub : 0;
-        int inw  = FF_CEIL_RSHIFT(inlink->w, hsub);
-        int inh  = FF_CEIL_RSHIFT(inlink->h, vsub);
-        int outw = FF_CEIL_RSHIFT(outlink->w, hsub);
-        int outh = FF_CEIL_RSHIFT(outlink->h, hsub);
+        const int outw = FF_CEIL_RSHIFT(outlink->w, hsub);
+        const int outh = FF_CEIL_RSHIFT(outlink->h, vsub);
+        ThreadData td = { .in = in,   .out  = out,
+                          .inw  = FF_CEIL_RSHIFT(inlink->w, hsub),
+                          .inh  = FF_CEIL_RSHIFT(inlink->h, vsub),
+                          .outh = outh, .outw = outw,
+                          .xi = -(outw-1) * c / 2, .yi =  (outw-1) * s / 2,
+                          .xprime = -(outh-1) * s / 2,
+                          .yprime = -(outh-1) * c / 2,
+                          .plane = plane, .c = c, .s = s };
 
-        const int xi = -outw/2 * c;
-        const int yi =  outw/2 * s;
-        int xprime = -outh/2 * s;
-        int yprime = -outh/2 * c;
-        int i, j, x, y;
 
-        for (j = 0; j < outh; j++) {
-            x = xprime + xi + FIXP*inw/2;
-            y = yprime + yi + FIXP*inh/2;
-
-            for (i = 0; i < outw; i++) {
-                int32_t v;
-                int x1, y1;
-                uint8_t *pin, *pout;
-                x += c;
-                y -= s;
-                x1 = x>>16;
-                y1 = y>>16;
-
-                /* the out-of-range values avoid border artifacts */
-                if (x1 >= -1 && x1 <= inw && y1 >= -1 && y1 <= inh) {
-                    uint8_t inp_inv[4]; /* interpolated input value */
-                    pout = out->data[plane] + j * out->linesize[plane] + i * rot->draw.pixelstep[plane];
-                    if (rot->use_bilinear) {
-                        pin = interpolate_bilinear(inp_inv,
-                                                   in->data[plane], in->linesize[plane], rot->draw.pixelstep[plane],
-                                                   x, y, inw-1, inh-1);
-                    } else {
-                        int x2 = av_clip(x1, 0, inw-1);
-                        int y2 = av_clip(y1, 0, inh-1);
-                        pin = in->data[plane] + y2 * in->linesize[plane] + x2 * rot->draw.pixelstep[plane];
-                    }
-                    switch (rot->draw.pixelstep[plane]) {
-                    case 1:
-                        *pout = *pin;
-                        break;
-                    case 2:
-                        *((uint16_t *)pout) = *((uint16_t *)pin);
-                        break;
-                    case 3:
-                        v = AV_RB24(pin);
-                        AV_WB24(pout, v);
-                        break;
-                    case 4:
-                        *((uint32_t *)pout) = *((uint32_t *)pin);
-                        break;
-                    default:
-                        memcpy(pout, pin, rot->draw.pixelstep[plane]);
-                        break;
-                    }
-                }
-            }
-            xprime += s;
-            yprime += c;
-        }
+        ctx->internal->execute(ctx, filter_slice, &td, NULL, FFMIN(outh, ctx->graph->nb_threads));
     }
 
     av_frame_free(&in);
@@ -441,7 +552,7 @@ static const AVFilterPad rotate_outputs[] = {
     { NULL }
 };
 
-AVFilter avfilter_vf_rotate = {
+AVFilter ff_vf_rotate = {
     .name          = "rotate",
     .description   = NULL_IF_CONFIG_SMALL("Rotate the input image."),
     .priv_size     = sizeof(RotContext),
@@ -452,5 +563,5 @@ AVFilter avfilter_vf_rotate = {
     .inputs        = rotate_inputs,
     .outputs       = rotate_outputs,
     .priv_class    = &rotate_class,
-    .flags         = AVFILTER_FLAG_SUPPORT_TIMELINE_GENERIC,
+    .flags         = AVFILTER_FLAG_SUPPORT_TIMELINE_GENERIC | AVFILTER_FLAG_SLICE_THREADS,
 };
