@@ -180,7 +180,7 @@ static int open_filter_param(FilterParam *f, int width, int height, unsigned int
     vec = sws_getGaussianVec(f->radius, f->quality);
     f->dist_width    = vec->length;
     f->dist_linesize = FFALIGN(vec->length, 8);
-    f->dist_coeff    = av_malloc(f->dist_width * f->dist_linesize * sizeof(*f->dist_coeff));
+    f->dist_coeff    = av_malloc_array(f->dist_width, f->dist_linesize * sizeof(*f->dist_coeff));
     if (!f->dist_coeff) {
         sws_freeVec(vec);
         return AVERROR(ENOMEM);
@@ -220,19 +220,6 @@ static int config_props(AVFilterLink *inlink)
 
 #define NB_PLANES 4
 
-static inline int mirror(int x, int w)
-{
-    if (!w)
-        return 0;
-
-    while ((unsigned)x > (unsigned)w) {
-        x = -x;
-        if (x < 0)
-            x += 2 * w;
-    }
-    return x;
-}
-
 static void blur(uint8_t       *dst, const int dst_linesize,
                  const uint8_t *src, const int src_linesize,
                  const int w, const int h, FilterParam *fp)
@@ -266,7 +253,8 @@ static void blur(uint8_t       *dst, const int dst_linesize,
                 for (dy = 0; dy < radius*2 + 1; dy++) {
                     int dx;
                     int iy = y+dy - radius;
-                    iy = mirror(iy, h-1);
+                    if      (iy < 0)  iy = -iy;
+                    else if (iy >= h) iy = h+h-iy-1;
 
                     for (dx = 0; dx < radius*2 + 1; dx++) {
                         const int ix = x+dx - radius;
@@ -277,11 +265,13 @@ static void blur(uint8_t       *dst, const int dst_linesize,
                 for (dy = 0; dy < radius*2+1; dy++) {
                     int dx;
                     int iy = y+dy - radius;
-                    iy = mirror(iy, h-1);
+                    if      (iy <  0) iy = -iy;
+                    else if (iy >= h) iy = h+h-iy-1;
 
                     for (dx = 0; dx < radius*2 + 1; dx++) {
                         int ix = x+dx - radius;
-                        ix = mirror(ix, w-1);
+                        if      (ix < 0)  ix = -ix;
+                        else if (ix >= w) ix = w+w-ix-1;
                         UPDATE_FACTOR;
                     }
                 }
