@@ -331,7 +331,7 @@ static int ogg_read_page(AVFormatContext *s, int *sid)
 
         c = avio_r8(bc);
 
-        if (url_feof(bc))
+        if (avio_feof(bc))
             return AVERROR_EOF;
 
         sync[sp++ & 3] = c;
@@ -796,7 +796,7 @@ retry:
         uint8_t *side_data = av_packet_new_side_data(pkt,
                                                      AV_PKT_DATA_SKIP_SAMPLES,
                                                      10);
-        if(side_data == NULL)
+        if(!side_data)
             goto fail;
         AV_WL32(side_data + 4, os->end_trimming);
         os->end_trimming = 0;
@@ -806,7 +806,7 @@ retry:
         uint8_t *side_data = av_packet_new_side_data(pkt,
                                                      AV_PKT_DATA_METADATA_UPDATE,
                                                      os->new_metadata_size);
-        if(side_data == NULL)
+        if(!side_data)
             goto fail;
 
         memcpy(side_data, os->new_metadata, os->new_metadata_size);
